﻿RepTracker = LibStub("AceAddon-3.0"):NewAddon("RepTracker", "AceConsole-3.0", "AceEvent-3.0")
local tracked

local defaults = {
	faction = {
		showInChat = true,
		dynamic = true,
		sticky = true,
		decrease = false,
		reset = true,
        trackTabard = false,
        dungeonsOnly = false,
		ZoneToFaction = {}
	}
}

local options = {
	name = "RepTracker",
	handler = RepTracker,
	type = "group",
	args = {
        version = {
			order = 1,
			type = 'description',
			name = 'Version: |cFFFF80802.5.1|r'
		},
		author = {
			order = 2,
			type = 'description',
			name = 'by |cFFFF8080InSo(v)nIaX|r'
		},
        trackheader = {
            order = 3,
            type = 'header',
            name = 'Tracking'
        },
		track = {
			order = 4,
			type = 'input',
			name = 'Current faction tracked:',
			desc = 'Sets which faction is tracked in this zone by default.  <none> to remove the default.',
			get = "GetZone",
			set = "changeBar",
            disabled = function() return RepTracker.db.faction.trackTabard end
		},
        trackTabard = {
            order = 5,
			type = 'toggle',
			name = 'Track Tabard',
			desc = 'Just track the reputation for the faction whose tabard you are wearing.',
			get = "Is",
			set = "Toggle"
        },
		trackDesc = {
			order = 6,
			type = 'description',
			name = 'Sets which faction is tracked in this zone by default.  <none> to remove the default.'
		},
		optionsHeader = {
			order = 7,
			type = 'header',
			name = 'Options'
		},
		dynamic = {
			order = 8,
			type = 'toggle',
			name = 'Dynamic Tracking',
			desc = 'The tracked reputation changes to reflect the last faction you gained reputation with.',
			get = "Is",
			set = "Toggle",
            disabled = function() return RepTracker.db.faction.trackTabard end
		},
		dynamicDesc = {
			order = 9,
			type = 'description',
			name = 'The tracked reputation changes to reflect the last faction you gained reputation with.'
		},
		sticky = {
			order = 10,
			type = 'toggle',
			name = 'Sticky Rep',
			desc = 'Dynamic tracking changes are permanently stored as the default tracked faction for that zone.',
			get = "Is",
			set = "Toggle",
			disabled = function()
                if not RepTracker.db.faction.dynamic or RepTracker.db.faction.trackTabard then
                    return true
                else
                    return false
                end
            end
		},
		stickyDesc = {
			order = 11,
			type = 'description',
			name = 'Dynamic tracking changes are permanently stored as the default tracked faction.',
            disabled = function() return RepTracker.db.faction.trackTabard end
		},
		showInChat = {
			order = 12,
			type = 'toggle',
			name = 'Show In Chat',
			desc = 'Display RepTracker messages in the chat window.',
			get = "Is",
			set = "Toggle"
		},
		showInChatDesc = {
			order = 13,
			type = 'description',
			name = 'Display RepTracker messages in the chat window.'
		},		
		decrease = {
			order = 14,
			type = 'toggle',
			name = 'Track Decreases',
			desc = 'Track reputation decreases.',
			get = "Is",
			set = "Toggle",
            disabled = function() return RepTracker.db.faction.trackTabard end
		},
		decreaseDesc = {
			order = 15,
			type = 'description',
			name = 'Track reputation decreases.'
		},
		reset = {
			order = 16,
			type = 'toggle',
			name = 'Faction Reset',
			desc = 'Clears the tracked faction upon entering a new zone if that zone does not yet have a faction assigned.',
			get = "Is",
			set = "Toggle",
            disabled = function() return RepTracker.db.faction.trackTabard end
		},
		resetDesc = {
			order = 17,
			type = 'description',
			name = 'Clears the tracked faction upon entering a new zone if that zone does not yet have a faction assigned.',
		}
	}
}

function RepTracker:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RepTrackerDB", defaults)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RepTracker", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RepTracker", "RepTracker")
	self:RegisterChatCommand("rt", "ChatCommand")
	self:RegisterChatCommand("RepTracker", "ChatCommand")
end

function RepTracker:OnEnable()
    CombatTextSetActiveUnit("player")
    if self.db.faction.dynamic and not self.db.faction.trackTabard then
        self:RegisterEvent("COMBAT_TEXT_UPDATE")
    end
    if self.db.faction.trackTabard then
        self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
        tracked = strlower(GetItemInfo(GetInventoryItemLink("player", 19)))
        self:ScanFactions()
        for i=1, getn(self.db.faction.tabardFactions) do
            if self.db.faction.tabardFactions[i] then
                if strmatch(tracked, self.db.faction.tabardFactions[i]) then
                    SetWatchedFactionIndex(i)
                    if self.db.faction.showInChat then
				        self:Print("Tracking "..self.db.faction.tabardFactions[i]:gsub("^%l", string.upper).." reputation")
			        end
                end
            end
        end
    else
        self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
    end
end

function RepTracker:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    elseif input == "track" then
        local name = GetWatchedFactionInfo()
        RepTracker:changeBar(_, name)
    elseif strmatch(input, "track") then
        RepTracker:changeBar(_, strtrim(input, "track "))
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(RepTracker, "rt", "RepTracker", input)
    end
end

function RepTracker:Is(info)
	return self.db.faction[info[1]]
end

function RepTracker:Toggle(info, value)
	self.db.faction[info[1]] = value
    if info[1] == "dynamic" and self.db.faction.dynamic then
        self:RegisterEvent("COMBAT_TEXT_UPDATE")
    elseif info[1] == "dynamic" then
        self:UnregisterEvent("COMBAT_TEXT_UPDATE")
    elseif info[1] == "trackTabard" and self.db.faction.trackTabard then
        self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
        self:UnregisterEvent("COMBAT_TEXT_UPDATE")
        self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
        tracked = strlower(GetItemInfo(GetInventoryItemLink("player", 19)))
        self:ScanFactions()
        for i=1, getn(self.db.faction.tabardFactions) do
            if self.db.faction.tabardFactions[i] then
                if strmatch(tracked, self.db.faction.tabardFactions[i]) then
                    SetWatchedFactionIndex(i)
                    if self.db.faction.showInChat then
				        self:Print("Tracking "..self.db.faction.tabardFactions[i]:gsub("^%l", string.upper).." reputation")
			        end
                    return
                end
            end
        end
    elseif info[1] == "trackTabard" then
        self:ZONE_CHANGED_NEW_AREA()
        self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
        self:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED")
        if self.db.faction.dynamic then
            self:RegisterEvent("COMBAT_TEXT_UPDATE")
        end
    end
end

function RepTracker:PLAYER_EQUIPMENT_CHANGED(_, slot, item)
    if slot == 19 and item == 1 then
        self:ScanFactions()
        tracked = strlower(GetItemInfo(GetInventoryItemLink("player", 19)))
        for i=1, getn(self.db.faction.tabardFactions) do
            if self.db.faction.tabardFactions[i] then
                if strmatch(tracked, self.db.faction.tabardFactions[i]) and not select(12, GetFactionInfo(i)) then
                    SetWatchedFactionIndex(i)
                    if self.db.faction.showInChat then
				        self:Print("Tracking "..self.db.faction.tabardFactions[i]:gsub("^%l", string.upper).." reputation")
			        end
                end
            end
        end
    end
end

function RepTracker:ScanFactions()
	local factions = {}
    local tabardFactions = {}
	local name, isHeader, isCollapsed
	factions["none"] = 0
	for i = 0, GetNumFactions() do
		_, _, _, _, _, _, _, _, isHeader, isCollapsed = GetFactionInfo(i)
		if isHeader and isCollapsed then
			ExpandFactionHeader(i)
		end
	end
	for i = 0, GetNumFactions() do
		name, _, _, _, _, _, _, _, isHeader = GetFactionInfo(i)
		if not isHeader and name ~= nil then
			factions[string.lower(name)] = i
            tabardFactions[i] = string.lower(name)
		end
	end
	self.db.faction.factions = factions
    self.db.faction.tabardFactions = tabardFactions
end

function RepTracker:ZONE_CHANGED_NEW_AREA()
	self:ScanFactions()
	local faction = self:GetZone()
	if faction == nil then
		if self.db.faction.showInChat then
			self:Print(GetZoneText()..": no faction tracked")
			self:Print("To set the tracked faction for this zone, type: /rt track <faction>")
		end
		if self.db.faction.reset then
			SetWatchedFactionIndex(0)
		end
	else
		faction = string.lower(faction)
		if self.db.faction.factions[faction] ~= nil then
			SetWatchedFactionIndex(self.db.faction.factions[faction])
			if self.db.faction.showInChat then
				self:Print(GetZoneText()..": tracking "..faction:gsub("^%l", string.upper).." reputation")
			end
		else
            if self.db.faction.showInChat then
			    self:Print("You have not made contact with "..faction:gsub("^%l", string.upper).." yet; tracking none")
            end
			if self.db.faction.reset then
				SetWatchedFactionIndex(0)
			end
		end
	end
end

function RepTracker:GetZone()
	return self.db.faction.ZoneToFaction[GetZoneText()]
end

function RepTracker:COMBAT_TEXT_UPDATE(_, messageType, faction, amount)
    if messageType ~= "FACTION" then
		return
	end
	if amount < 0 and self.db.faction.dynamic and self.db.faction.decrease then
		self:changeBar(_, faction)
	elseif amount > 0 and self.db.faction.dynamic then
		self:changeBar(_, faction)
	end
end

function RepTracker:changeBar(_, faction)
    if faction == nil or faction == "none" then
        if self.db.faction.showInChat and self.db.faction.ZoneToFaction[GetZoneText()] ~= nil then
			self:Print("Not tracking a reputation")
        end
        SetWatchedFactionIndex(0)
        self.db.faction.ZoneToFaction[GetZoneText()] = nil
        return
    end
    self:ScanFactions()
	local oldName = strlower(GetWatchedFactionInfo())
	for i=1, GetNumFactions() do
		local name = strlower(GetFactionInfo(i))
		if name == strlower(faction) and name ~= oldName and name ~= "alliance" and name ~= "horde" then
			SetWatchedFactionIndex(i)
			if self.db.faction.showInChat then
                local function tc(f, r)
                    return f:upper()..r:lower()
                end
				self:Print("Now tracking "..name:gsub("(%a)([%w_']*)", tc).." reputation")
			end
			if self.db.faction.sticky then
				self.db.faction.ZoneToFaction[GetZoneText()] = string.lower(faction)
	        end
            return
		end
	end
end